@@ -71,25 +71,35 @@ class UpgradeTestBackendQemu(UpgradeTestBackend):
         self.qemu_pid = None
         self.profiledir = os.path.dirname(profile)
         # setup mount dir/imagefile location
-        self.baseimage = self.config.get("NonInteractive","BaseImage")
+        self.baseimage = self.config.get("KVM","BaseImage")
         if not os.path.exists(self.baseimage):
             raise NoImageFoundException
-        if self.config.getWithDefault("NonInteractive","SwapImage",""):
+        # check if we want virtio here and default to yes
+        try:
+            virtio = self.config.getboolean("KVM","Virtio")
+        except ConfigParser.NoOptionError,e:
+            virtio = True
+        if virtio:
+            self.qemu_options.extend(["-net","nic,model=virtio"])
+            self.qemu_options.extend(["-net","user"])
+        # swapimage
+        if self.config.getWithDefault("KVM","SwapImage",""):
             self.qemu_options.append("-hdb")
-            self.qemu_options.append(self.config.get("NonInteractive","SwapImage"))
+            self.qemu_options.append(self.config.get("KVM","SwapImage"))
+        # regular image
         self.image = os.path.join(self.profiledir, "test-image")
         # make ssh login possible (localhost 54321) available
         self.ssh_key = os.path.join(self.profiledir,self.config.getWithDefault("NonInteractive","SSHKey","ssh-key"))
-        self.ssh_port = self.config.getWithDefault("NonInteractive","SshPort","54321")
+        self.ssh_port = self.config.getWithDefault("KVM","SshPort","54321")
         self.qemu_options.append("-redir")
         self.qemu_options.append("tcp:%s::22" % self.ssh_port)
         # vnc port/display
-        vncport = self.config.getWithDefault("NonInteractive","VncNum","0")
+        vncport = self.config.getWithDefault("KVM","VncNum","0")
         self.qemu_options.append("-vnc")
         self.qemu_options.append("localhost:%s" % vncport)
 
         # make the memory configurable
-        mem = self.config.getWithDefault("NonInteractive","VirtualRam","768")
+        mem = self.config.getWithDefault("KVM","VirtualRam","768")
         self.qemu_options.append("-m")
         self.qemu_options.append(str(mem))
 
@@ -98,13 +108,15 @@ class UpgradeTestBackendQemu(UpgradeTestBackend):
                            shell=True) == 0:
             raise PortInUseException, "the port is already in use (another upgrade tester is running?)"
 
-    def _copyToImage(self, fromF, toF):
+    def _copyToImage(self, fromF, toF, recursive=False):
         cmd = ["scp",
                "-P",self.ssh_port,
                "-q","-q", # shut it up
                "-i",self.ssh_key,
                "-o", "StrictHostKeyChecking=no",
                "-o", "UserKnownHostsFile=%s" % os.path.dirname(self.profile)+"/known_hosts"]
+        if recursive:
+            cmd.append("-r")
         # we support both single files and lists of files
         if isinstance(fromF,list):
             cmd += fromF
@@ -134,7 +146,7 @@ class UpgradeTestBackendQemu(UpgradeTestBackend):
         # ssh -l root -p 54321 localhost -i profile/server/ssh_key
         #     -o StrictHostKeyChecking=no
         ret = subprocess.call(["ssh",
-                               "-tt",
+#                               "-tt",
                                "-l","root",
                                "-p",self.ssh_port,
                                "localhost",
@@ -148,9 +160,11 @@ class UpgradeTestBackendQemu(UpgradeTestBackend):
 
     def installPackages(self, pkgs):
         " install additional pkgs (list) into the vm before the ugprade "
+        if not pkgs:
+            return True
         self.start()
         self._runInImage(["apt-get","update"])
-        ret = self._runInImage(["DEBIAN_FRONTEND=noninteractive","apt-get","install", "-y"]+pkgs)
+        ret = self._runInImage(["DEBIAN_FRONTEND=noninteractive","apt-get","install", "--reinstall", "-y"]+pkgs)
         self.stop()
         return (ret == 0)
 
@@ -182,7 +196,7 @@ class UpgradeTestBackendQemu(UpgradeTestBackend):
         #        way to do this currently without running as root
         # as a workaround we regenerate manually every now and then
         # and use UpgradeFromDistOnBootstrap=true here
-        self.config.set("NonInteractive","CacheBaseImage", "false")
+        self.config.set("KVM","CacheBaseImage", "false")
         self.config.set("NonInteractive","UpgradeFromDistOnBootstrap","true")
         self.baseimage = "jeos/%s-i386.qcow2" % self.config.get("Sources","To")
         self.image = diff_image
@@ -206,13 +220,18 @@ class UpgradeTestBackendQemu(UpgradeTestBackend):
     def bootstrap(self, force=False):
         print "bootstrap()"
 
+        # move old crash files away so that test() is not
+        # confused by them
+        for f in glob.glob(self.resultdir+"/*.crash"):
+            shutil.move(f, f+".old")
+
         # copy image into place, use baseimage as template
         # we expect to be able to ssh into the baseimage to
         # set it up
         if (not force and
             os.path.exists("%s.%s" % (self.image,self.fromDist)) and 
-            self.config.has_option("NonInteractive","CacheBaseImage") and
-            self.config.getboolean("NonInteractive","CacheBaseImage")):
+            self.config.has_option("KVM","CacheBaseImage") and
+            self.config.getboolean("KVM","CacheBaseImage")):
             print "Not bootstraping again, we have a cached BaseImage"
             shutil.copy("%s.%s" % (self.image,self.fromDist), self.image)
             return True
@@ -223,6 +242,7 @@ class UpgradeTestBackendQemu(UpgradeTestBackend):
         # get common vars
         mirror = self.config.get("NonInteractive","Mirror")
         basepkg = self.config.get("NonInteractive","BasePkg")
+        additional_base_pkgs = self.config.getlist("Distro","BaseMetaPkgs")
 
         # start the VM
         self.start()
@@ -278,17 +298,18 @@ iface eth0 inet static
         # FIXME: instead of this retrying (for network errors with 
         #        proxies) we should have a self._runAptInImage() 
         for i in range(3):
-            ret = self._runInImage(["DEBIAN_FRONTEND=noninteractive","apt-get","install", "-y",basepkg])
+            ret = self._runInImage(["DEBIAN_FRONTEND=noninteractive","apt-get","install", "-y",basepkg]+additional_base_pkgs)
         assert(ret == 0)
 
         CMAX = 4000
         pkgs =  self.config.getListFromFile("NonInteractive","AdditionalPkgs")
         while(len(pkgs)) > 0:
             print "installing additonal: %s" % pkgs[:CMAX]
-            ret= self._runInImage(["DEBIAN_FRONTEND=noninteractive","apt-get","install","-y"]+pkgs[:CMAX])
+            ret= self._runInImage(["DEBIAN_FRONTEND=noninteractive","apt-get","install","--reinstall","-y"]+pkgs[:CMAX])
             print "apt(2) returned: %s" % ret
             if ret != 0:
                 #self._cacheDebs(tmpdir)
+                self.stop()
                 return False
             pkgs = pkgs[CMAX+1:]
 
@@ -308,18 +329,18 @@ iface eth0 inet static
             print "running apt-get upgrade in from dist (after bootstrap)"
             for i in range(3):
                 ret = self._runInImage(["DEBIAN_FRONTEND=noninteractive","apt-get","-y","dist-upgrade"])
-            assert(ret == 0)
+            assert(ret == 0, "dist-upgrade returned %s" % ret)
 
         print "Cleaning image"
         ret = self._runInImage(["apt-get","clean"])
-        assert(ret == 0)
+        assert(ret == 0, "apt-get clean returned %s" % ret)
 
         # done with the bootstrap
         self.stop()
 
         # copy cache into place (if needed)
-        if (self.config.has_option("NonInteractive","CacheBaseImage") and
-            self.config.getboolean("NonInteractive","CacheBaseImage")):
+        if (self.config.has_option("KVM","CacheBaseImage") and
+            self.config.getboolean("KVM","CacheBaseImage")):
             shutil.copy(self.image, "%s.%s" % (self.image,self.fromDist))
         
         return True
@@ -327,21 +348,29 @@ iface eth0 inet static
     def saveVMSnapshot(self,name):
         # savevm
         print "savevm"
-        self.qemu_pid.stdin.write("stop\n")
-        self.qemu_pid.stdin.write("savevm %s\n" % name)
-        self.qemu_pid.stdin.write("cont\n")
+        self.stop()
+        shutil.copy(self.image, self.image+"."+name)
+        return
+        # *sigh* buggy :/
+        #self.qemu_pid.stdin.write("stop\n")
+        #self.qemu_pid.stdin.write("savevm %s\n" % name)
+        #self.qemu_pid.stdin.write("cont\n")
     def delVMSnapshot(self,name):
         print "delvm"
         self.qemu_pid.stdin.write("delvm %s\n" % name)
     def restoreVMSnapshot(self,name):
         print "restorevm"
+        self.stop()
+        shutil.copy(self.image+"."+name, self.image)
+	return
         # loadvm
-        self.qemu_pid.stdin.write("stop\n")
-        self.qemu_pid.stdin.write("loadvm %s\n" % name)
-        self.qemu_pid.stdin.write("cont\n")
+        # *sigh* buggy :/
+        #self.qemu_pid.stdin.write("stop\n")
+        #self.qemu_pid.stdin.write("loadvm %s\n" % name)
+        #self.qemu_pid.stdin.write("cont\n")
 
     def start(self):
-        print "Starting qemu"
+        print "Starting %s %s" % (self.qemu_binary, self.qemu_options)
         if self.qemu_pid != None:
             print "already runing"
             return True
@@ -387,18 +416,26 @@ iface eth0 inet static
         print "copy upgrader into image"
         # copy the upgrade into target+/upgrader-tester/
         files = []
-        self._runInImage(["mkdir","/upgrade-tester"])
+        self._runInImage(["mkdir","-p","/upgrade-tester","/etc/update-manager/release-upgrades.d"])
         for f in glob.glob("%s/DistUpgrade/*" % self.basefilesdir):
             if not os.path.isdir(f):
                 files.append(f)
+            elif os.path.islink(f):
+                print "Copying link '%s' to image " % f
+                self._copyToImage(f, "/upgrade-tester", recursive=True)
         self._copyToImage(files, "/upgrade-tester")
+        # copy the patches
+        d="%s/DistUpgrade/patches" % self.basefilesdir
+        print "Copying '%s' to image" % d
+        self._copyToImage(d, "/upgrade-tester/", recursive=True)
         # copy the profile
         if os.path.exists(self.profile):
-            print "Copying '%s' to image " % self.profile
-            self._copyToImage(self.profile, "/upgrade-tester")
+            print "Copying '%s' to image overrides" % self.profile
+            self._copyToImage(self.profile, "/etc/update-manager/release-upgrades.d/")
         # and any other cfg files
         for f in glob.glob(os.path.dirname(self.profile)+"/*.cfg"):
-            if os.path.isfile(f):
+            if (os.path.isfile(f) and
+                not os.path.basename(f).startswith("DistUpgrade.cfg")):
                 print "Copying '%s' to image " % f
                 self._copyToImage(f, "/upgrade-tester")
         # and prereq lists
@@ -449,7 +486,7 @@ iface eth0 inet static
         print "Shuting down the VM"
         self.stop()
 
-        return True
+        return (ret == 0)
 
     def test(self):
         # FIXME: add some tests here to see if the upgrade worked
@@ -460,6 +497,14 @@ iface eth0 inet static
         # - generate diff of upgrade vs fresh install
         # ...
         #self.genDiff()
+        self.start()
+        self._copyFromImage("/var/crash/*.crash", self.resultdir)
+        crashfiles = glob.glob(self.resultdir+"/*.crash")
+        self.stop()
+        if len(crashfiles) > 0:
+            print "WARNING: crash files detected on the upgrade"
+            print crashfiles
+            return False
         return True
         
 
